/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import static java.util.stream.Collectors.*;

import java.util.List;
import java.util.Map;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import lombok.Builder;

public class AttachmentTestFactory {

	public static final String XML_FILE_NAME = "XML-Daten-1.xml";

	public static final String XML_CONTENT_TYPE = "text/xml";
	public static final String XML_CONTENT_STRING = """
			<?xml version="1.0" encoding="UTF-8"?>
			<myForm xmlns:pdf="http://xmlns.cit.de/assistants/pdf"
				xmlns:t="http://xmlns.cit.de/intelliform/transaction"
				t:id="20201118365670866101\" t:timestamp=\"2020-11-18T09:09:27.627Z"
				t:sender="afm.schleswig-holstein.de"
				t:form="Kleiner Waffenschein gem. § 10 Abs. 4 Satz 4 Waffengesetz (WaffG)"
				t:form-id="waffen/kleinerWaffenschein"
				t:customer="Einheitlicher Ansprechpartner" t:customer-id="ea-sh"
				t:client="Schleswig-Holstein"
				t:client-id="land">
				  <Upload1>
				    <file content-type="image/png" description="" id="VendorId3333" length="155251">Image.png</file>
			      </Upload1>
			    </myForm>""";
	public static final String XML_CONTENT = XML_CONTENT_STRING;
	public static final String XML_ATTACHMENT_ID = "myForm-xml";
	public static final String XML_NAME = "XML-Daten.xml";

	public static final String PDF_ATTACHMENT_CONTENT_TYPE = "application/pdf";
	public static final String PDF_ATTACHMENT_CONTENT = "TestContent2";
	public static final String PDF_ATTACHMENT_NAME = "Scan1.pdf";
	public static final String PDF_ATTACHMENT_ID = "VendorId2222";
	public static final String PDF2_ATTACHMENT_ID = "VendorIdpdf2";

	public static final String PNG_ATTACHMENT_CONTENT_TYPE = "application/pdf";
	public static final String PNG_ATTACHMENT_CONTENT = "TestContent3";
	public static final String PNG_ATTACHMENT_NAME = "Image.png";
	public static final String PNG_ATTACHMENT_ID = "VendorId3333";

	public static final String JPG_ATTACHMENT_ID = "VendorIdjpg1";
	public static final String ODT_ATTACHMENT_ID = "VendorIdodt1";

	public static final String ODT_ATTACHMENT_CONTENT_TYPE = "application/vnd.oasis.opendocument.text";

	public static final String DOCX_ATTACHMENT_CONTENT_TYPE = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
	public static final String DOCX_ATTACHMENT_CONTENT = "TestContent4";

	public static final String DOCX1_ATTACHMENT_ID = "VendorIddocx1";
	public static final String DOCX2_ATTACHMENT_ID = "VendorIddocx2";
	public static final String DOCX3_ATTACHMENT_ID = "VendorIddocx3";
	public static final String DOCX4_ATTACHMENT_ID = "VendorIddocx4";
	public static final String DOCX5_ATTACHMENT_ID = "VendorIddocx5";
	public static final String DOCX6_ATTACHMENT_ID = "VendorIddocx6";
	public static final String DOCX7_ATTACHMENT_ID = "VendorIddocx7";
	public static final String DOCX8_ATTACHMENT_ID = "VendorIddocx8";
	public static final String ORGANISATIONSEINHEITEN_ID = "10363455";

	public static final String ANTRAGSTELLER_ANREDE = "Herr";
	public static final String ANTRAGSTELLER_ANREDE_CODE = "03";
	public static final String ANTRAGSTELLER_NACHNAME = "Mustermann";

	public static final String ATTRIBUTES_ENTRY_KEY = "X-IntelliForm-Signed";
	public static final String ATTRIBUTES_ENTRY_VALUE = "false";

	public static Attachment createXmlDaten() {
		return createXmlDatenWithContent(XML_CONTENT);
	}

	public static Attachment createPdf() {
		return createAttachment(MetaAttachment.builder()
				.id(PDF_ATTACHMENT_ID)
				.name(PDF_ATTACHMENT_NAME)
				.contentType(PDF_ATTACHMENT_CONTENT_TYPE)
				.content(PDF_ATTACHMENT_CONTENT)
				.build());
	}

	public static Attachment createPng() {
		return createAttachment(MetaAttachment.builder()
				.id(PNG_ATTACHMENT_ID)
				.name(PNG_ATTACHMENT_NAME)
				.contentType(PNG_ATTACHMENT_CONTENT_TYPE)
				.content(PNG_ATTACHMENT_CONTENT)
				.build());
	}

	private static Attachment createAttachment(MetaAttachment metaAttachment) {
		var attachment = new Attachment();
		attachment.getAttributes().add(createAttributesEntry());
		attachment.setContent(metaAttachment.content.getBytes());
		attachment.setContentType(metaAttachment.contentType);
		attachment.setId(metaAttachment.id);
		attachment.setName(metaAttachment.name);
		return attachment;
	}

	private static Entry createAttributesEntry() {
		var attributesEntry = new Entry();
		attributesEntry.setKey(ATTRIBUTES_ENTRY_KEY);
		attributesEntry.setValue(ATTRIBUTES_ENTRY_VALUE);
		return attributesEntry;
	}

	public static List<Attachment> createManyAttachments(String xmlTemplateString, Map<String, String> templateValues,
			String... attachmentParameterMatrix) {

		var metaAttachments = arrayToMetaAttachments(attachmentParameterMatrix);

		var allTemplateValues = Stream.concat(
				templateValues.entrySet().stream(),
				metaAttachments.stream().map(metaAttachment -> Map.entry(metaAttachment.templateId, metaAttachment.id))
		).collect(toMap(Map.Entry::getKey, Map.Entry::getValue));

		var xmlFormString = fillTemplateString(
				xmlTemplateString,
				allTemplateValues
		);

		return Stream.concat(
				Stream.of(createXmlDatenWithContent(xmlFormString)),
				metaAttachments.stream().map(AttachmentTestFactory::createAttachment)
		).toList();
	}

	private static Attachment createXmlDatenWithContent(String content) {
		return createAttachment(MetaAttachment.builder()
				.id(XML_ATTACHMENT_ID)
				.name(XML_NAME)
				.contentType(XML_CONTENT_TYPE)
				.content(content)
				.build()
		);
	}

	@Builder
	private record MetaAttachment(String id, String name, String contentType, String content, String templateId) {
	}

	private static List<MetaAttachment> arrayToMetaAttachments(String[] array) {
		int parameterCount = 5;
		if (array.length % parameterCount != 0) {
			throw new IllegalArgumentException("Matrix length must be divisible by %d.".formatted(parameterCount));
		}

		return IntStream.range(0, array.length / parameterCount)
				.boxed()
				.map(row -> row * parameterCount)
				.map(offset -> new MetaAttachment(array[offset], array[offset + 1], array[offset + 2], array[offset + 3], array[offset + 4]))
				.toList();
	}

	private static String fillTemplateString(String templateString, Map<String, String> templateValues) {
		return templateValues.entrySet().stream()
				.reduce(templateString,
						(currentString, entry) -> currentString.replace("${" + entry.getKey() + "}", entry.getValue()),
						(s1, s2) -> s1);
	}
}
